@@ -23,7 +23,7 @@ QWaylandXdgOutputManagerV1::QWaylandXdgOutputManagerV1(QWaylandDisplay* display,
 }
 
 QWaylandScreen::QWaylandScreen(QWaylandDisplay *waylandDisplay, int version, uint32_t id)
-    : QtWayland::wl_output(waylandDisplay->wl_registry(), id, qMin(version, 2))
+    : QtWayland::wl_output(waylandDisplay->wl_registry(), id, qMin(version, 3))
     , m_outputId(id)
     , mWaylandDisplay(waylandDisplay)
     , mOutputName(QStringLiteral("Screen%1").arg(id))
@@ -44,6 +44,8 @@ QWaylandScreen::~QWaylandScreen()
 {
     if (zxdg_output_v1::isInitialized())
         zxdg_output_v1::destroy();
+    if (wl_output::isInitialized() && wl_output::version() >= WL_OUTPUT_RELEASE_SINCE_VERSION)
+        wl_output::release();
 }
 
 uint QWaylandScreen::requiredEvents() const
@@ -161,6 +163,16 @@ QList<QPlatformScreen *> QWaylandScreen::virtualSiblings() const
     return list;
 }
 
+QWindow *QWaylandScreen::topLevelAt(const QPoint & pos) const
+{
+    if (QWaylandWindow::fixedToplevelPositions) {
+        Q_UNUSED(pos);
+        return nullptr;
+    }
+
+    return QPlatformScreen::topLevelAt(pos);
+}
+
 Qt::ScreenOrientation QWaylandScreen::orientation() const
 {
     return m_orientation;
@@ -188,6 +200,32 @@ QPlatformCursor *QWaylandScreen::cursor() const
 }
 #endif // QT_CONFIG(cursor)
 
+QPlatformScreen::SubpixelAntialiasingType QWaylandScreen::subpixelAntialiasingTypeHint() const
+{
+    QPlatformScreen::SubpixelAntialiasingType type = QPlatformScreen::subpixelAntialiasingTypeHint();
+    if (type == QPlatformScreen::Subpixel_None) {
+        switch (mSubpixel) {
+        case wl_output::subpixel_unknown:
+        case wl_output::subpixel_none:
+            type = QPlatformScreen::Subpixel_None;
+            break;
+        case wl_output::subpixel_horizontal_rgb:
+            type = QPlatformScreen::Subpixel_RGB;
+            break;
+        case wl_output::subpixel_horizontal_bgr:
+            type = QPlatformScreen::Subpixel_BGR;
+            break;
+        case wl_output::subpixel_vertical_rgb:
+            type = QPlatformScreen::Subpixel_VRGB;
+            break;
+        case wl_output::subpixel_vertical_bgr:
+            type = QPlatformScreen::Subpixel_VBGR;
+            break;
+        }
+    }
+    return type;
+}
+
 QWaylandScreen *QWaylandScreen::waylandScreenFromWindow(QWindow *window)
 {
     QPlatformScreen *platformScreen = QPlatformScreen::platformScreenForWindow(window);
@@ -203,6 +241,35 @@ QWaylandScreen *QWaylandScreen::fromWlOutput(::wl_output *output)
     return nullptr;
 }
 
+Qt::ScreenOrientation QWaylandScreen::toScreenOrientation(int wlTransform,
+                                                          Qt::ScreenOrientation fallback) const
+{
+    auto orientation = fallback;
+    bool isPortrait = mGeometry.height() > mGeometry.width();
+    switch (wlTransform) {
+    case WL_OUTPUT_TRANSFORM_NORMAL:
+        orientation = isPortrait ? Qt::PortraitOrientation : Qt::LandscapeOrientation;
+        break;
+    case WL_OUTPUT_TRANSFORM_90:
+        orientation = isPortrait ? Qt::InvertedLandscapeOrientation : Qt::PortraitOrientation;
+        break;
+    case WL_OUTPUT_TRANSFORM_180:
+        orientation = isPortrait ? Qt::InvertedPortraitOrientation : Qt::InvertedLandscapeOrientation;
+        break;
+    case WL_OUTPUT_TRANSFORM_270:
+        orientation = isPortrait ? Qt::LandscapeOrientation : Qt::InvertedPortraitOrientation;
+        break;
+    // Ignore these ones, at least for now
+    case WL_OUTPUT_TRANSFORM_FLIPPED:
+    case WL_OUTPUT_TRANSFORM_FLIPPED_90:
+    case WL_OUTPUT_TRANSFORM_FLIPPED_180:
+    case WL_OUTPUT_TRANSFORM_FLIPPED_270:
+        break;
+    }
+
+    return orientation;
+}
+
 void QWaylandScreen::output_mode(uint32_t flags, int width, int height, int refresh)
 {
     if (!(flags & WL_OUTPUT_MODE_CURRENT))
@@ -223,11 +290,10 @@ void QWaylandScreen::output_geometry(int32_t x, int32_t y,
                                      const QString &model,
                                      int32_t transform)
 {
-    Q_UNUSED(subpixel);
-
     mManufacturer = make;
     mModel = model;
 
+    mSubpixel = subpixel;
     mTransform = transform;
 
     mPhysicalSize = QSize(width, height);
@@ -255,29 +321,11 @@ void QWaylandScreen::output_done()
 void QWaylandScreen::updateOutputProperties()
 {
     if (mTransform >= 0) {
-        bool isPortrait = mGeometry.height() > mGeometry.width();
-        switch (mTransform) {
-            case WL_OUTPUT_TRANSFORM_NORMAL:
-                m_orientation = isPortrait ? Qt::PortraitOrientation : Qt::LandscapeOrientation;
-                break;
-            case WL_OUTPUT_TRANSFORM_90:
-                m_orientation = isPortrait ? Qt::InvertedLandscapeOrientation : Qt::PortraitOrientation;
-                break;
-            case WL_OUTPUT_TRANSFORM_180:
-                m_orientation = isPortrait ? Qt::InvertedPortraitOrientation : Qt::InvertedLandscapeOrientation;
-                break;
-            case WL_OUTPUT_TRANSFORM_270:
-                m_orientation = isPortrait ? Qt::LandscapeOrientation : Qt::InvertedPortraitOrientation;
-                break;
-            // Ignore these ones, at least for now
-            case WL_OUTPUT_TRANSFORM_FLIPPED:
-            case WL_OUTPUT_TRANSFORM_FLIPPED_90:
-            case WL_OUTPUT_TRANSFORM_FLIPPED_180:
-            case WL_OUTPUT_TRANSFORM_FLIPPED_270:
-                break;
+        auto newOrientation = toScreenOrientation(mTransform, m_orientation);
+        if (m_orientation != newOrientation) {
+            m_orientation = newOrientation;
+            QWindowSystemInterface::handleScreenOrientationChange(screen(), m_orientation);
         }
-
-        QWindowSystemInterface::handleScreenOrientationChange(screen(), m_orientation);
         mTransform = -1;
     }
 
